'''File for handling all operations relating to the Procedure resource'''

import uuid
import random
from fhir.resources.procedure import Procedure

from fhirgenerator.helpers.helpers import makeRandomDate


def generateProcedure(resource_detail: dict, patient_id: str, start_date: str, days: int) -> dict:
    '''Generate Procedure resource from resource detail from configuration'''

    procedure_id = str(uuid.uuid4())

    procedure_code = random.choice(resource_detail['codes'])

    random_date = makeRandomDate(start_date, days)

    procedure_data = {
        'id': procedure_id,
        'code': {
            'coding': [
                procedure_code
            ]
        },
        'status': 'completed',
        'subject': {
            'reference': f'Patient/{patient_id}'
        },
        'performedDateTime': random_date
    }

    procedure_resource = Procedure(**procedure_data).dict()
    return procedure_resource



'''File for handling all operations relating to the Observation resource'''

import uuid
import random
from fhir.resources.observation import Observation

from fhirgenerator.helpers.helpers import makeRandomDate


def generateObservation(resource_detail: dict, patient_id: str, start_date: str, days: int) -> dict:
    '''Generate Observation Resource from resource detail from configuration'''

    observation_id = str(uuid.uuid4())

    observation_code = random.choice(resource_detail['codes'])

    random_date = makeRandomDate(start_date, days)

    if 'enumSetList' in resource_detail:
        enum_set_list = resource_detail['enumSetList']
        if isinstance(enum_set_list[0], dict):
            if 'system' in enum_set_list[0]:
                value_x_type = 'CodeableConcept'
                value_x_value = random.choice(enum_set_list)
            elif 'value' in enum_set_list[0]:
                value_x_type = 'Quantity'
                value_x_value = random.choice(enum_set_list)
        elif len(enum_set_list[0].split(':')) > 1:
            value_x_type = 'Ratio'
            value_x_titer_choice = random.choice(enum_set_list)
            value_x_titer_choice_split = value_x_titer_choice.split(':')
            value_x_value = {
                'numerator': {'value': value_x_titer_choice_split[0]},
                'denominator': {'value': value_x_titer_choice_split[1]}
            }
        elif enum_set_list[0].isnumeric():
            value_x_type = 'Integer'
            value_x_value = random.choice(enum_set_list)
        else:
            value_x_type = 'String'
            value_x_value = random.choice(enum_set_list)
    elif 'minValue' in resource_detail and 'maxValue' in resource_detail:
        min_value = resource_detail['minValue']
        max_value = resource_detail['maxValue']
        if 'decimalValue' in resource_detail:
            decimal_value = resource_detail['decimalValue']
            value_x_type = 'Quantity'
            value_x_value = {
                'value': round(random.uniform(min_value, max_value), decimal_value)
            }
            if decimal_value == 0:
                value_x_value['value'] = float(value_x_value['value'])
            if 'unit' in resource_detail:
                system, code, display = resource_detail['unit'].split('^')
                value_x_value['unit'] = display
                value_x_value['system'] = system
                value_x_value['code'] = code
        else:
            if 'unit' in resource_detail:
                value_x_type = 'Quantity'
                system, code, display = resource_detail['unit'].split('^')
                value_x_value = {
                    'value': round(random.uniform(min_value, max_value)),
                    'unit': display,
                    'system': system,
                    'code': code
                }
            else:
                value_x_type = 'Integer'
                value_x_value = round(random.uniform(min_value, max_value))
    else:
        print("Warning: There was no enumSetList or (minValue and maxValue) in your configuration for this Observation. This Observation will not have a value[x].")
        value_x_type = 'None'
        value_x_value = ''

    observation_data = {
        'id': observation_id,
        'status': 'final',
        'code': {
            'coding': [
                observation_code
            ]
        },
        'subject': {
            'reference': f'Patient/{patient_id}'
        },
        'effectiveDateTime': str(random_date),
        f'value{value_x_type}': value_x_value
    }

    if 'valueNone' in observation_data:
        del observation_data['valueNone']

    observation_resource = Observation(**observation_data).dict()
    return observation_resource



'''Testing Cndition generator'''

import orjson
from dateutil import parser
import datetime
from fhirgenerator.resources.r4.condition import generateCondition
from fhirgenerator.helpers.helpers import default


def testConditionGenerator():
    '''Test function for Condition Generator'''
    with open('fhirgenerator/tests/input/config.json', 'rb') as infile:
        config_dict = orjson.loads(infile.read())

    patient_id = '26774-827647-736278-3737646'

    condition_resource_details = []
    for detail in config_dict['resourceDetails']:
        if detail['fhirResource'] == 'Condition':
            condition_resource_details.append(detail)

    for i, detail in enumerate(condition_resource_details):
        created_resource = generateCondition(detail, patient_id, config_dict['startDate'], config_dict['days'])

        assert created_resource['resourceType'] == 'Condition'
        assert created_resource['code']['coding'][0] in detail['codes']
        assert (created_resource['onsetDateTime'] >= parser.parse(config_dict['startDate'])) and (created_resource['onsetDateTime'] <= parser.parse(config_dict['startDate']) + datetime.timedelta(days=config_dict['days']))
        assert created_resource['subject']['reference'] == f'Patient/{patient_id}'

        with open(f'fhirgenerator/tests/output/test_condition_{i}.json', 'wb') as outfile:
            outfile.write(orjson.dumps(created_resource, default=default, option=orjson.OPT_NAIVE_UTC))



'''File for handling all operations relating to the ContactPoint datatype'''

import random
from faker import Faker
from fhir.resources.contactpoint import ContactPoint


def generateContactPoint(system: str = 'phone') -> dict:
    '''Function for generating a FHIR ContactPoint'''

    fake = Faker()

    contact_point_data = {
        'system': system,
        'use': random.choice(['home', 'work', 'mobile'])
    }

    if system == 'phone':
        contact_point_data['value'] = fake.phone_number()
    elif system == 'email':
        contact_point_data['value'] = fake.free_email()

    contact_point = ContactPoint(**contact_point_data).dict()

    return contact_point



'''Testing Encounter generator'''

import orjson
from dateutil import parser
import datetime
from fhirgenerator.resources.r4.encounter import generateEncounter, class_options
from fhirgenerator.helpers.helpers import default


def testEncounterGenerator():
    '''Test function for Encounter Generator'''

    with open('fhirgenerator/tests/input/config.json', 'rb') as infile:
        config_dict = orjson.loads(infile.read())

    patient_id = '26774-827647-736278-3737646'

    encounter_resource_details = []
    for detail in config_dict['resourceDetails']:
        if detail['fhirResource'] == 'Encounter':
            encounter_resource_details.append(detail)

    for i, detail in enumerate(encounter_resource_details):
        created_resource = generateEncounter(detail, patient_id, config_dict['startDate'], config_dict['days'])

        assert created_resource['resourceType'] == 'Encounter'
        assert created_resource['status'] == 'finished'
        assert created_resource['class'] in class_options
        assert created_resource['type'][0]['coding'][0] in detail['codes']
        assert (created_resource['period']['start'] >= parser.parse(config_dict['startDate'])) and (created_resource['period']['start'] <= parser.parse(config_dict['startDate']) + datetime.timedelta(days=config_dict['days']))
        assert created_resource['subject']['reference'] == f'Patient/{patient_id}'

        with open(f'fhirgenerator/tests/output/test_encounter_{i}.json', 'wb') as outfile:
            outfile.write(orjson.dumps(created_resource, default=default, option=orjson.OPT_NAIVE_UTC))



'''File for handling all operations relating to the MedicationStatement resource'''

import uuid
import random
from fhir.resources.medicationstatement import MedicationStatement

from fhirgenerator.helpers.helpers import makeRandomDate
from fhirgenerator.types.dosage import generateDosage


def generateMedicationStatement(resource_detail: dict, patient_id: str, start_date: str, days: int) -> dict:
    '''Generate MedicationStatement resource from resource detail from configuration'''

    ms_id = str(uuid.uuid4())

    medication_cc = random.choice(resource_detail['codes'])

    random_date = makeRandomDate(start_date, days)

    try:
        if 'enumSetList' in resource_detail:
            enum_set_list = resource_detail['enumSetList']
            dosage = generateDosage(enum_set_list=enum_set_list)
        else:
            if 'decimalValue' in resource_detail:
                dosage = generateDosage(min_value=resource_detail['minValue'], max_value=resource_detail['maxValue'], decimal_value=resource_detail['decimalValue'], unit_str=resource_detail['unit'])
            else:
                dosage = generateDosage(min_value=resource_detail['minValue'], max_value=resource_detail['maxValue'], unit_str=resource_detail['unit'])
    except KeyError:
        raise NameError('There was no enumSetList in your configuration and there is no listed unit. Without an enumSetList, you MUST have the unit key in your configuration.')

    ms_data = {
        'id': ms_id,
        'status': 'completed',
        'medicationCodeableConcept': {'coding': [medication_cc]},
        'effectiveDateTime': random_date,
        'subject': {
            'reference': f'Patient/{patient_id}'
        },
        'dosage': [dosage]
    }

    medicationstatement_resource = MedicationStatement(**ms_data).dict()
    return medicationstatement_resource



'''File for handling all operations relating to the Address datatype'''

import random
from faker import Faker
from fhir.resources.address import Address


def generateAddress(use: str = 'home') -> dict:
    '''Function for generating a FHIR Address'''

    address_data = {
        'use': use,
        'line': [generateLineAddress()],
        'city': generateCityAddress(),
        'state': generateStateAddress(),
        'postalCode': generatePostalCodeAddress()
    }

    address = Address(**address_data).dict()
    return address


def generateLineAddress():
    '''Function for generating a line in a FHIR Address'''
    fake = Faker()
    return fake.street_address()


def generateCityAddress():
    '''Function for generating a city in a FHIR Address'''
    fake = Faker()
    return fake.city()


def generateStateAddress():
    '''Function for generating a state in a FHIR Address'''
    fake = Faker()
    return fake.state_abbr()


def generatePostalCodeAddress():
    '''Function for generating a postal code in a FHIR Address'''
    rand_str = str(random.randint(1, 99950))
    if len(rand_str) != 5:
        leading_zeros = ''.join(['0' for x in range(0, 5 - len(rand_str))])
        rand_str = leading_zeros + rand_str
    return rand_str



'''File for handling all operations relating to the Condition resource'''

import uuid
import random
from fhir.resources.condition import Condition

from fhirgenerator.helpers.helpers import makeRandomDate


def generateCondition(resource_detail: dict, patient_id: str, start_date: str, days: int) -> dict:
    '''Generate Condition Resource from resource detail from configuration'''

    condition_id = str(uuid.uuid4())

    condition_code = random.choice(resource_detail['codes'])

    random_date = makeRandomDate(start_date, days)

    condition_data = {
        'id': condition_id,
        'code': {
            'coding': [
                condition_code
            ]
        },
        'subject': {
            'reference': f'Patient/{patient_id}'
        },
        'onsetDateTime': str(random_date)
    }

    condition_resource = Condition(**condition_data)
    return dict(condition_resource.dict())



'''Testing Patient generator'''

import orjson
from fhirgenerator.resources.r4.bundle import generateBundle
from fhirgenerator.helpers.helpers import default


def testBundleGenerator():
    '''Test function for Bundle Generator'''

    patient_id = '26774-827647-736278-3737646'
    fake_resources = [
        {
            'resourceType': 'Patient',
            'id': patient_id
        },
        {
            'resourceType': 'Observation',
            'id': '73736464647474',
            'status': 'final',
            'code': {'coding': [{'code': '6457-9'}]}
        },
        {
            'resourceType': 'Condition',
            'id': '7373737383838383',
            'subject': {'reference': f'Patient/{patient_id}'}
        },
        {
            'resourceType': 'Observation',
            'id': '7373738929292727',
            'status': 'final',
            'code': {'coding': [{'code': '6467-9'}]}
        },
    ]
    created_resource_collection = generateBundle(fake_resources)
    created_resource_transaction = generateBundle(fake_resources, type='transaction')

    assert created_resource_transaction['type'] == 'transaction'
    assert created_resource_transaction['entry'][0]['resource']['resourceType'] == 'Patient'
    assert created_resource_transaction['entry'][0]['resource']['id'] == patient_id

    assert created_resource_transaction['entry'][1]['resource']['resourceType'] == 'Observation'
    assert created_resource_transaction['entry'][1]['resource']['id'] == '73736464647474'
    assert created_resource_transaction['entry'][1]['resource']['status'] == 'final'
    assert created_resource_transaction['entry'][1]['resource']['code']['coding'][0]['code'] == '6457-9'

    assert created_resource_transaction['entry'][2]['resource']['resourceType'] == 'Condition'
    assert created_resource_transaction['entry'][2]['resource']['id'] == '7373737383838383'
    assert created_resource_transaction['entry'][2]['resource']['subject']['reference'] == f'Patient/{patient_id}'

    assert created_resource_transaction['entry'][3]['resource']['resourceType'] == 'Observation'
    assert created_resource_transaction['entry'][3]['resource']['id'] == '7373738929292727'
    assert created_resource_transaction['entry'][3]['resource']['status'] == 'final'
    assert created_resource_transaction['entry'][3]['resource']['code']['coding'][0]['code'] == '6467-9'

    assert created_resource_transaction['entry'][0]['request']['method'] == 'POST'
    assert created_resource_transaction['entry'][0]['request']['url'] == 'Patient'

    assert created_resource_transaction['entry'][1]['request']['method'] == 'POST'
    assert created_resource_transaction['entry'][1]['request']['url'] == 'Observation'

    assert created_resource_transaction['entry'][2]['request']['method'] == 'POST'
    assert created_resource_transaction['entry'][2]['request']['url'] == 'Condition'

    assert created_resource_transaction['entry'][3]['request']['method'] == 'POST'
    assert created_resource_transaction['entry'][3]['request']['url'] == 'Observation'

    assert created_resource_collection['type'] == 'collection'

    assert created_resource_collection['entry'][0]['resource']['resourceType'] == 'Patient'
    assert created_resource_collection['entry'][0]['resource']['id'] == patient_id

    assert created_resource_collection['entry'][1]['resource']['resourceType'] == 'Observation'
    assert created_resource_collection['entry'][1]['resource']['id'] == '73736464647474'
    assert created_resource_collection['entry'][1]['resource']['status'] == 'final'
    assert created_resource_collection['entry'][1]['resource']['code']['coding'][0]['code'] == '6457-9'

    assert created_resource_collection['entry'][2]['resource']['resourceType'] == 'Condition'
    assert created_resource_collection['entry'][2]['resource']['id'] == '7373737383838383'
    assert created_resource_collection['entry'][2]['resource']['subject']['reference'] == f'Patient/{patient_id}'

    assert created_resource_collection['entry'][3]['resource']['resourceType'] == 'Observation'
    assert created_resource_collection['entry'][3]['resource']['id'] == '7373738929292727'
    assert created_resource_collection['entry'][3]['resource']['status'] == 'final'
    assert created_resource_collection['entry'][3]['resource']['code']['coding'][0]['code'] == '6467-9'

    with open('fhirgenerator/tests/output/test_bundle_transaction.json', 'wb') as outfile:
        outfile.write(orjson.dumps(created_resource_transaction, default=default, option=orjson.OPT_NAIVE_UTC))
    with open('fhirgenerator/tests/output/test_bundle_collection.json', 'wb') as outfile:
        outfile.write(orjson.dumps(created_resource_collection, default=default, option=orjson.OPT_NAIVE_UTC))



'''Model for Configuration Dictionary'''

import datetime

configuration_dictionary_template = {
    'numberPatients': 10,
    'ageMin': 0,
    'ageMax': 99,
    'genderMFOU': [47, 47, 1, 5],
    'startDate': str(datetime.datetime.today()),
    'days': 365,
    'outputFolder': 'output',
    'resourceDetails': {
        'fhirResource': 'Resource',
        'codesystem': '',
        'codes': '',
        'codesetURL': '',
        'minValue': 0,
        'maxValue': 10,
        'decimalValue': 1,
        'enumSetList': '',
        'minOccurrencesPerCycle': 1,
        'maxOccurrencesPerCycle': 1,
        'cycleLengthInDays': 365
    }
}



'''File for handling all operations relating to the Patient resource'''

import uuid
import random
import datetime
from dateutil import parser
from fhir.resources.patient import Patient
from fhirgenerator.types.contactPoint import generateContactPoint

from fhirgenerator.types.humanName import generateName
from fhirgenerator.types.address import generateAddress
from fhirgenerator.types.identifier import generateMRNIdentifier

gender_map = {
    'M': 'male',
    'F': 'female',
    'O': 'other',
    'U': 'unknown'
}


def generateBirthDateFromAge(age: int, startDate: str) -> str:
    '''Generate a random birthday that matches the given age'''
    start = parser.parse(startDate)
    random_number_of_days = random.randrange(365)
    birthdate = datetime.date(start.year - age - 1, start.month, start.day) + datetime.timedelta(days=random_number_of_days)
    return str(birthdate)


def generatePatient(configuration: dict) -> dict:
    '''Generate Patient Resource'''

    patient_id = str(uuid.uuid4())
    patient_data = {
        'id': patient_id,
        'identifier': [generateMRNIdentifier()],
        'name': [generateName(gender=configuration['gender'], use='official')],
        'address': [generateAddress(use='home')],
        'telecom': [generateContactPoint(system='phone')],
        'gender': gender_map[configuration['gender'].upper()],
        'birthDate': generateBirthDateFromAge(configuration['age'], configuration['startDate'])
    }

    # Handles if a Patient will have a usual or maiden name
    if random.choices([True, False], weights=[40, 60]):  # 40% chance of having a maiden name
        patient_data['name'].append(generateName(gender=configuration['gender'], use='maiden'))
    elif random.choices([True, False], weights=[60, 40]):  # 60% chance of having a usual name
        patient_data['name'].append(generateName(gender=configuration['gender'], use='usual'))

    # Handles if a Patient will have an old address
    if random.choices([True, False], weights=[25, 75]):
        patient_data['address'].append(generateAddress(use='old'))

    # Handles if a Patient will have an email address
    if random.choices([True, False], weights=[90, 10]):
        patient_data['telecom'].append(generateContactPoint(system='email'))

    patient_resource = Patient(**patient_data).dict()
    return patient_resource



'''File for handling all operations relating to the US Core Condition resource'''

from fhirgenerator.resources.r4.condition import generateCondition


def generateUSCoreCondition(detail: dict, patient_id: str, start_date: str, days: str):
    '''Generate a US Core Condition'''

    condition_resource = generateCondition(detail, patient_id, start_date, days)
    condition_resource['meta'] = {'profile': ['http://hl7.org/fhir/us/core/StructureDefinition/us-core-condition']}
    condition_resource['clinicalStatus'] = {'coding': [{'system': 'http://terminology.hl7.org/CodeSystem/condition-clinical', 'code': 'active', 'display': 'Active'}]}
    condition_resource['category'] = [{'coding': [{'system': 'http://terminology.hl7.org/CodeSystem/condition-category', 'code': 'problem-list-item'}]}]
    return condition_resource



# -*- coding: utf-8 -*-
"""
Profile: http://hl7.org/fhir/us/core/StructureDefinition/us-core-patient
Release: STU4
Version: 4.0.0
"""

import typing

from pydantic import validator, Field

from fhir.resources.patient import Patient
from fhir.resources import fhirtypes


class USCorePatient(Patient):
    '''Defines constraints and extensions on the patient resource for the minimal set of data to query and retrieve patient demographic information.'''

    gender: fhirtypes.Code = Field(
        ...,
        alias="gender",
        title="male | female | other | unknown",
        description=(
            "Administrative Gender - the gender that the patient is considered to "
            "have for administration and record keeping purposes."
        ),
        # if property is element of this resource.
        element_property=True,
        # note: Enum values can be used in validation,
        # but use in your own responsibilities, read official FHIR documentation.
        enum_values=["male", "female", "other", "unknown"],
    )

    identifier: typing.List[fhirtypes.IdentifierType] = Field(
        ...,
        alias="identifier",
        title="An identifier for this patient",
        description=None,
        # if property is element of this resource.
        element_property=True,
    )

    name: typing.List[fhirtypes.HumanNameType] = Field(
        ...,
        alias="name",
        title="A name associated with the patient",
        description="A name associated with the individual.",
        # if property is element of this resource.
        element_property=True,
    )

    @validator('identifier')
    def check_identifier(cls, identifiers):
        '''Identifier 1..*, Identifier.system 1..1, Identifier.value 1..1'''
        for identifier in identifiers:
            if identifier.system is None and identifier.value is None:
                raise ValueError('system and value are required')
            elif identifier.system is None:
                raise ValueError('system is required')
            elif identifier.value is None:
                raise ValueError('value is required')
        return identifiers

    @validator('telecom')
    def check_telecom(cls, telecoms):
        '''telecom 0..*, telecom.system 1..1, telecom.value 1..1'''
        for telecom in telecoms:
            if telecom.system is None and telecom.value is None:
                raise ValueError('system and value are required')
            elif telecom.system is None:
                raise ValueError('system is required')
            elif telecom.value is None:
                raise ValueError('value is required')
        return telecoms



'''File for handling all operations relating to the Identifer datatype'''

from faker import Faker


def generateMRNIdentifier() -> dict:
    '''Generate a Patient.identifier for a MRN'''
    fake = Faker()
    identifier_dict = {
        'system': 'urn:fhirgen:mrn',
        'type': {'coding': [
            {
                'system': 'http://terminology.hl7.org/CodeSystem/v2-0203',
                'code': 'MR'
            }
        ]},
        'value': fake.ssn()
    }
    return identifier_dict



'''File for handling all operations relating to the HumanName datatype'''

from faker import Faker


def generateName(gender: str = 'U', use: str = 'official'):
    '''Function to generate a FHIR HumanName based on gender and use'''
    human_name = {
        'use': use,
        'family': generateFamilyName(),
        'given': [generateGivenName(gender)]
    }

    if use == 'maiden':
        del human_name['given']
    elif use == 'usual':
        del human_name['family']

    return human_name


def generateFamilyName():
    '''Function to generate family name'''
    fake = Faker()
    return fake.last_name()


def generateGivenName(gender: str = 'U'):
    '''Function to generate given name'''
    fake = Faker()
    if gender == 'M':
        return fake.first_name_male()
    if gender == 'F':
        return fake.first_name_female()
    return fake.first_name_nonbinary()



'''Testing Observation generator'''

import orjson
from dateutil import parser
import datetime
from fhirgenerator.resources.r4.observation import generateObservation
from fhirgenerator.helpers.helpers import default


def testObservationGenerator():
    '''Test function for Observation Generator'''

    with open('fhirgenerator/tests/input/config.json', 'rb') as infile:
        config_dict = orjson.loads(infile.read())

    patient_id = '26774-827647-736278-3737646'

    observation_resource_details = []
    for detail in config_dict['resourceDetails']:
        if detail['fhirResource'] == 'Observation':
            observation_resource_details.append(detail)

    for i, detail in enumerate(observation_resource_details):

        if 'enumSetList' in detail:
            enum_set_list = detail['enumSetList']
            if len(enum_set_list[0].split(':')) > 1:
                value_x_type = 'Ratio'
            elif 'system' in enum_set_list[0]:
                value_x_type = 'CodeableConcept'
            elif 'value' in enum_set_list[0]:
                value_x_type = 'Quantity'
            else:
                value_x_type = 'String'
        elif 'decimalValue' in detail:
            value_x_type = 'Quantity'
        else:
            value_x_type = 'Integer'

        created_resource = generateObservation(detail, patient_id, config_dict['startDate'], config_dict['days'])

        assert created_resource['resourceType'] == 'Observation'
        assert created_resource['status'] == 'final'
        assert created_resource['code']['coding'][0] in detail['codes']
        assert (created_resource['effectiveDateTime'] >= parser.parse(config_dict['startDate'])) and (created_resource['effectiveDateTime'] <= parser.parse(config_dict['startDate']) + datetime.timedelta(days=config_dict['days']))
        assert created_resource['subject']['reference'] == f'Patient/{patient_id}'

        match value_x_type:
            case ['Ratio']:
                assert created_resource['valueRatio']['numerator'] + ':' + created_resource['valueRatio']['denominator'] in detail['enumSetList']
            case ['CodeableConcept']:
                assert created_resource['valueCodeableConcept']['coding'][0] in detail['enumSetList']
            case ['Quantity']:
                if 'enumSetList' in detail:
                    assert created_resource['valueQuantity'] in detail['enumSetList']
                else:
                    assert created_resource['valueQuantity']['value'] <= detail['maxValue'] and created_resource['valueQuantity']['value'] >= detail['minValue']
                    test_system, test_code, test_unit = detail['unit'].split('^')
                    assert created_resource['valueQuantity']['system'] == test_system
                    assert created_resource['valueQuantity']['code'] == test_code
                    assert created_resource['valueQuantity']['unit'] == test_unit
            case ['String']:
                assert created_resource['valueString'] in detail['enumSetList']
            case ['Integer']:
                if 'enumSetList' in detail:
                    assert created_resource['valueInteger'] in detail['enumSetList']
                else:
                    assert created_resource['valueInteger'] <= detail['maxValue'] and created_resource['valueInteger'] >= detail['minValue']

        with open(f'fhirgenerator/tests/output/test_observation_{i}.json', 'wb') as outfile:
            outfile.write(orjson.dumps(created_resource, default=default, option=orjson.OPT_NAIVE_UTC))


# TODO: write more observation tests for all possible value[x] that the package supports



'''Setup file for fhirgenerator package'''
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requirements = ["fhir.resources>=6.2.2", "Faker>=13.3.4", "orjson>=3.6.8"]

setuptools.setup(
    name="fhirgenerator",
    version="0.1.0",
    author="Andrew Stevens",
    author_email="andrew.stevens@gtri.gatech.edu",
    description="A package to generate FHIR Resources using a configuration file",
    install_requires=requirements,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SmartChartSuite/FHIRGenerator",
    project_urls={
        "Bug Tracker": "https://github.com/SmartChartSuite/FHIRGenerator/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.10",
    package_data={'fhirgenerator.tests.input': ['config.json']},
    entry_points={
        'console_scripts': ['fhirgenerator=fhirgenerator.command_line:main']
    }
)



'''Helper functions file'''

import datetime
from dateutil import parser
import decimal
import random


def calculateAge(birthDate: datetime.datetime, start: str = str(datetime.date.today())) -> int:
    '''Calculate Age from birth date'''
    today = parser.parse(start)
    age = today.year - birthDate.year - ((today.month, today.day) < (birthDate.month, birthDate.day))
    return age


def default(obj):
    '''Callable function for JSON serialization of non-supported types'''
    if isinstance(obj, decimal.Decimal):
        if float(str(obj)).is_integer():
            return int(float(str(obj)))
        else:
            return float(str(obj))
    raise TypeError


def makeRandomDate(start_date: str, days: int):
    '''Make random date using a start date and a number of days'''
    start_date = parser.parse(start_date)
    random_number_of_days = random.randrange(days)
    random_date = start_date + datetime.timedelta(days=random_number_of_days)
    return random_date
